import compileall
import os
import shutil
import fire
import python_minifier


class CompileMinify(object):
    """Minify, compile and remove python files"""

    def __init__(self, directory: str = os.getcwd()):
        if not os.path.exists(directory):
            raise Exception('The directory "{}" was not found'.format(directory))
        self.directory = directory

    def run(
        self,
        remove_tests: bool = False,
        remove_py: bool = False,
        compile_root_py: bool = False,
    ):
        self._apply_to_python_files(
            self._minify_and_compile,
            remove_tests=remove_tests,
            remove_py=remove_py,
            compile_root_py=compile_root_py,
        )
        print("Python files minified and compiled")

    def _apply_to_python_files(self, function, **kwargs):
        # No compile python files in root folder
        # to preserve entrypoint
        level = 0
        for r, d, f in os.walk(self.directory):
            if r.endswith("tests") and kwargs["remove_tests"]:
                shutil.rmtree(r)
            elif level > 0 or kwargs["compile_root_py"]:
                for file in f:
                    if file.endswith(".py"):
                        file = os.path.join(r, file)
                        function(file)
                        if kwargs["remove_py"] and not file.endswith("__init__.py"):
                            os.remove(file)
            level += 1

    @staticmethod
    def _minify_and_compile(file):
        with open(file) as f:
            buff = f.read()
        with open(file, "w") as f:
            mnf = python_minifier.minify(buff)
            f.write(mnf)
        compileall.compile_file(file, force=True, quiet=True, legacy=True)


def entrypoint():
    fire.Fire(CompileMinify)



from setuptools import setup, find_packages
from setuptools.command.install import install
import os
import sys


NAME = "compile-minifier"
VERSION = "0.1.3"


def readme():
    """print long description"""
    with open("README.md") as f:
        return f.read()


with open("requirements/base.txt") as f:
    REQUIRES = f.read().strip().split("\n")

with open("requirements/ci.txt") as f:
    CI_REQUIRES = f.read().strip().split("\n")


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""

    description = "verify that the git tag matches our version"

    def run(self):
        tag = os.getenv("CIRCLE_TAG")

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


setup(
    name=NAME,
    version=VERSION,
    description="Python compiler and minifier",
    long_description=readme(),
    long_description_content_type="text/markdown",
    author_email="contact@bimdata.io",
    url="https://github.com/bimdata/compile-minifier",
    install_requires=REQUIRES,
    extras_require={"ci": CI_REQUIRES,},
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.5, <4",
    include_package_data=True,
    entry_points={"console_scripts": ["compileminify=compileminify.main:entrypoint",],},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    cmdclass={"verify": VerifyVersionCommand,},
)
